@@ -8,7 +8,7 @@ import hashlib
 import time
 import uuid
 from enum import Enum
-from typing import List
+from typing import List, Dict
 
 import requests
 
@@ -47,6 +47,7 @@ class PropertyIDs(Enum):
     BRIGHTNESS = "P1501"  # From 0-100
     COLOR_TEMP = "P1502"  # In Kelvin
     COLOR = "P1507"  # As a hex string RrGgBb
+    DOOR_OPEN = "P2001"  # 0 if the door is closed
 
 
 class ResponseCodes(Enum):
@@ -55,6 +56,10 @@ class ResponseCodes(Enum):
     ACCESS_TOKEN_ERROR = "2001"
 
 
+class ResponseCodesLock(Enum):
+    SUCCESS = 0
+
+
 SWITCHABLE_DEVICES = [DeviceTypes.LIGHT, DeviceTypes.MESH_LIGHT, DeviceTypes.PLUG]
 
 
@@ -72,6 +77,9 @@ class ParameterError(Exception):
 class AccessTokenError(Exception):
     pass
 
+class LoginError(Exception):
+    pass
+
 
 class UnknownApiError(Exception):
     def __init__(self, response_json):
@@ -111,11 +119,16 @@ class BaseClient:
 
         response_json = requests.post("https://auth-prod.api.wyze.com/user/login",
                                       headers=headers, json=login_payload).json()
+
+        # response_json = requests.post("https://auth-prod.api.wyze.com/user/login",
+        #                               json=login_payload).json()
+
         try:
             self.access_token = response_json['access_token']
             self.refresh_token = response_json['refresh_token']
             return True
-        except KeyError:
+        except KeyError as e:
+            print(e)
             return False
 
     def can_login(self, username, password):
@@ -130,6 +143,16 @@ class BaseClient:
     @staticmethod
     def check_for_errors(response_json):
         if response_json['code'] != ResponseCodes.SUCCESS.value:
+            if response_json['code'] == ResponseCodes.PARAMETER_ERROR.value:
+                raise ParameterError(response_json)
+            elif response_json['code'] == ResponseCodes.ACCESS_TOKEN_ERROR.value:
+                raise AccessTokenError
+            else:
+                raise UnknownApiError(response_json)
+
+    @staticmethod
+    def check_for_errors_lock(response_json):
+        if response_json['ErrNo'] != ResponseCodes.SUCCESS.value:
             if response_json['code'] == ResponseCodes.PARAMETER_ERROR.value:
                 raise ParameterError
             elif response_json['code'] == ResponseCodes.ACCESS_TOKEN_ERROR.value:
@@ -137,6 +160,41 @@ class BaseClient:
             else:
                 raise UnknownApiError(response_json)
 
+    app_key = "275965684684dbdaf29a0ed9"
+    app_secret = "4deekof1ba311c5c33a9cb8e12787e8c"
+
+    def postMethod(self, str, str2, map):
+        str4 = str
+        str3 = str2
+        map2 = map
+
+        return self.getTransferParam(map2, str3, str4, "post")
+
+    def getTransferParam(self, map, str4, str2, str3):
+        map["accessToken"] = self.access_token
+        map["key"] = self.app_key
+        map["timestamp"] = str(datetime.datetime.now().timestamp()).split(".")[0] + "000"
+        map["sign"] = self.getSignStr(str4, str3, map)
+        import json
+        print(json.dumps(map))
+        return map
+
+    def getSignStr(self, str, str2, map: Dict):
+        string_buf = str2 + str
+        for entry in sorted(map.keys()):
+            print(entry)
+            print(map[entry])
+            string_buf += entry + "=" + map[entry] + "&"
+
+        string_buf = string_buf[:-1]
+        print(string_buf)
+        string_buf += self.app_secret
+        import urllib.parse
+        urlencoded = urllib.parse.quote_plus(string_buf)
+        sign_str = hashlib.md5(urlencoded.encode()).hexdigest()
+        print(sign_str)
+        return sign_str
+
     def get_object_list(self):
         payload = {
             "phone_system_type": PHONE_SYSTEM_TYPE,
@@ -153,6 +211,7 @@ class BaseClient:
         response_json = requests.post("https://api.wyzecam.com/app/v2/home_page/get_object_list",
                                       json=payload).json()
 
+        print(payload)
         self.check_for_errors(response_json)
 
         return response_json
@@ -168,11 +227,12 @@ class BaseClient:
             "access_token": self.access_token,
             "phone_id": PHONE_ID,
             "app_name": APP_NAME,
-            "device_list": [device.mac],
-            "target_pid_list": ["P3", "P5", "P1501", "P1502", "P1506", "P1507", "P1509", "P1511", "P1512"]
+            "device_model": device.product_model,
+            "device_mac": device.mac,
+            "target_pid_list": []
         }
 
-        response_json = requests.post("https://api.wyzecam.com/app/v2/device_list/get_property_list",
+        response_json = requests.post("https://api.wyzecam.com/app/v2/device/get_property_list",
                                       json=payload).json()
 
         self.check_for_errors(response_json)
@@ -316,4 +376,22 @@ class BaseClient:
 
         self.check_for_errors(response_json)
 
-        return response_json
\ No newline at end of file
+        return response_json
+
+    def lock_control(self, device: Device, action: str):
+        sb2 = "https://yd-saas-toc.wyzecam.com/openapi/lock/v1/control"
+        str3 = "/openapi/lock/v1/control"
+
+        uuid = device.mac.split(".")[2]
+
+        hash_map = {
+            "uuid": uuid,
+            "action": action  # "remoteLock" or "remoteUnlock"
+        }
+
+        payload = self.postMethod(sb2, str3, hash_map)
+
+        url = "https://yd-saas-toc.wyzecam.com/openapi/lock/v1/control"
+
+        response_json = requests.post(url, json=payload).json()
+        print(response_json)
