@@ -117,6 +117,10 @@ class Client:
                 plist.extend(extra_pids)
 
             self.client.run_action_list(device, plist)
+        elif device_type in [
+            DeviceTypes.LOCK
+        ]:
+            self.client.lock_control(device, "remoteLock")
         else:
             raise ActionNotSupported(device_type.value)
 
@@ -148,6 +152,10 @@ class Client:
                 plist.extend(extra_pids)
 
             self.client.run_action_list(device, plist)
+        elif device_type in [
+            DeviceTypes.LOCK
+        ]:
+            self.client.lock_control(device, "remoteUnlock")
         else:
             raise ActionNotSupported(device_type.value)
 
@@ -179,7 +187,7 @@ class Client:
         ])
 
     def get_info(self, device):
-        properties = self.client.get_property_list(device)['data']['device_list'][0]['device_property_list']
+        properties = self.client.get_property_list(device)['data']['property_list']
 
         property_list = []
         for property in properties:
